import {EditorState, Plugin} from 'prosemirror-state'
import {EditorView} from 'prosemirror-view'
import {history} from 'prosemirror-history'
import {keymap} from 'prosemirror-keymap'
import {baseKeymap} from 'prosemirror-commands'
import {DOMParser, DOMSerializer} from 'prosemirror-model'

import {Menu, blocks, marks} from './godo-menus.mjs'
import {dialog} from './godo-dialog.mjs'
import {loadSchema, buildKeymap} from './utils.mjs'
import base from './components/base.mjs'
import fontMarks from './components/font-marks.mjs'
import heading from './components/heading.mjs'
import link from './components/link.mjs'
import list from './components/list.mjs'

function menuPlugin (menu) {
  return new Plugin({
    view (editorView) {
      let menuView = new Menu(menu, editorView)
      editorView.dom.parentNode.insertBefore(menuView.el, editorView.dom)
      return menuView
    },
  })
}

function changeEventPlugin (editor) {
  return new Plugin({
    state: {
      init () {},
      apply (tr, callback) {
        if (tr.docChanged) {
          clearTimeout(callback)
          callback = setTimeout(() => {
            editor.textarea.value = editor.getHTML(tr.doc.content)
            editor.textarea.dispatchEvent(new Event('change', { 'bubbles': true }))
          }, 300)
        }
        return callback
      },
    },
  })
}

export default class Godo extends window.HTMLElement {
  static observedAttributes = ['no-edit']

  get value () {
    return this.getHTML(this.state.doc.content)
  }

  set value (newValue) {
    this._resetState(newValue)
    this.textarea.value = this.value
    this.view.updateState(this.state)
  }

  connectedCallback () {
    const initialContent = this.innerHTML
    this.innerHTML = ''
    this.textarea = document.createElement('textarea')
    this.textarea.id = this.id
    this.textarea.hidden = true
    this.textarea.style.display = 'none'
    this.textarea.setAttribute('name', this.getAttribute('name'))
    this.appendChild(this.textarea)

    const options = {
      schema: this.getAttribute('schema') || 'full',
      instantUpdate: this.getAttribute('instant-update') !== null,
      updateEvent: this.getAttribute('update-event') || false,
    }

    if (this.getAttribute('heading-levels')) {
      options.headingLevels = this.getAttribute('heading-levels')
    }

    const componentsDefinitions = [base, fontMarks, link]
    if (options.schema === 'full') {
      componentsDefinitions.push(heading, list)
    }

    const components = componentsDefinitions.map(c => c(options))
    this.schema = loadSchema(components)

    this.editorWrapper = document.createElement('div')
    this.editorWrapper.className = 'godo'
    this.textarea.insertAdjacentElement('afterend', this.editorWrapper)

    this.marksMenu = menuPlugin(marks(components, this.schema))
    this.blocksMenu = menuPlugin(blocks(components, this.schema))
    this.menuDialog = document.body.appendChild(dialog)

    this.pluginsList = [
      history(),
      keymap(buildKeymap(components, this.schema, null)),
      keymap(baseKeymap),
      this.marksMenu,
    ]

    if (options.schema !== 'basic' && !this.textarea.readOnly) this.pluginsList.push(this.blocksMenu)
    if (options.instantUpdate) this.pluginsList.push(changeEventPlugin(this))

    this._resetState(initialContent)
    this.textarea.value = this.value

    const label = this.id ? document.querySelector(`label[for=${this.id}]`) : undefined
    this.view = new EditorView(this.editorWrapper, {
      state: this.state,
      editable: () => !this.readOnly && !this.getAttribute('no-edit'),
      attributes: {
        class: 'godo--editor',
        role: 'textbox',
        'aria-multiline': 'true',
        'aria-labelledby': label ? label.id : undefined,
      },
    })

    if (this.textarea.form) {
      this.textarea.form.addEventListener('submit', () => {
        this.textarea.value = this.value
      })
    }

    const updateEvent = options.updateEvent
    if (updateEvent) {
      this.textarea.addEventListener(updateEvent, () => {
        this.value = this.textarea.value
      })
    }
  }

  attributeChangedCallback (name) {
    if (name === 'no-edit') {
      this.view.updateState(this.state)
    }
  }

  _resetState (htmlContent) {
    const tempDiv = document.createElement('div')
    tempDiv.innerHTML = htmlContent
    this.state = EditorState.create({
      schema: this.schema,
      doc: DOMParser.fromSchema(this.schema).parse(tempDiv),
      plugins: this.pluginsList,
    })
  }

  getHTML (from) {
    const div = document.createElement('div')
    const fragment = DOMSerializer
      .fromSchema(this.schema)
      .serializeFragment(from)
    div.appendChild(fragment)
    return div.innerHTML
  }
}

window.customElements.define('godo-editor', Godo)

document.addEventListener('DOMContentLoaded', function () {
  const godoTextareas = document.querySelectorAll('textarea[data-godo-schema]')
  godoTextareas.forEach((textarea) => {
    const godoEditor = document.createElement('godo-editor')
    godoEditor.id = textarea.id
    godoEditor.setAttribute('name', textarea.getAttribute('name'))
    godoEditor.readOnly = textarea.readOnly
    godoEditor.setAttribute('schema', textarea.dataset.godoSchema)
    if (textarea.dataset.godoInstantUpdate !== undefined) {
      godoEditor.setAttribute('instant-update', textarea.dataset.godoInstantUpdate)
    }
    godoEditor.setAttribute('update-event', textarea.dataset.godoUpdateEvent)
    godoEditor.dataSet

    if (textarea.dataset.godoHeadingLevels) {
      godoEditor.setAttribute('heading-levels', textarea.dataset.godoHeadingLevels)
    }
    godoEditor.innerHTML = textarea.value
    textarea.replaceWith(godoEditor)
  })
})

