import {expect} from 'vitest'
import { domTest } from './helpers.mjs'
import '../src-js/godo.js'

domTest('legacy textarea attributes are set on godo element', async ({appendToDom}) => {
  const dom = appendToDom(`
    <textarea
      id="identifier"
      name="field_name"
      data-godo-schema="full"
      data-godo-instant-update
      data-godo-update-event="update_event"
      data-godo-heading-levels="1,2">
      <p>Initial content</p>
    </textarea>
  `)

  document.dispatchEvent(new Event('DOMContentLoaded', {
    bubbles: true,
    cancelable: true,
  }))

  const godo = dom.querySelector('godo-editor')
  expect(godo).not.toBe(null)
  expect(godo.id).toBe('identifier')
  expect(godo.getAttribute('name')).toBe('field_name')
  expect(godo.getAttribute('schema')).toBe('full')
  expect(godo.getAttribute('instant-update')).not.toBe(null)
  expect(godo.getAttribute('update-event')).toBe('update_event')
  expect(godo.getAttribute('heading-levels')).toBe('1,2')
  expect(godo.querySelector('p').innerHTML).toBe('Initial content')
})

domTest('name attribute is set on textarea', async ({appendToDom}) => {
  const godo = appendToDom(`
    <godo-editor name="field_name">
    </godo-editor>
  `)
  const textarea = godo.querySelector('textarea')
  expect(textarea.getAttribute('name')).toBe('field_name')
})

domTest('initial content is moved to textarea', async ({appendToDom}) => {
  // invalid-tag to check initial content is parsed before being assigned to
  // the text area value
  const godo = appendToDom(`
    <godo-editor>
      <invalid-tag>Initial</invalid-tag>
      <p>Content</p>
    </godo-editor>
  `)
  const textarea = godo.querySelector('textarea')
  expect(textarea.value).toBe('<p>Initial</p><p>Content</p>')
  // should've been removed
  expect(godo.querySelector('invalid-tag')).toBe(null)
})

domTest('no-edit attribute sets ProseMirror contenteditable', async ({appendToDom}) => {
  const dom = appendToDom('<godo-editor></godo-editor>')
  const editor = dom.querySelector('godo-editor')
  expect(editor.view.editable).toBe(true)
  editor.setAttribute('no-edit', true)
  expect(editor.view.editable).toBe(false)
  editor.removeAttribute('no-edit')
  expect(editor.view.editable).toBe(true)
})

domTest('value property get and sets HTML content', async ({appendToDom}) => {
  const dom = appendToDom(`
    <godo-editor>
      <invalid-tag>Initial content</invalid-tag>
    </godo-editor>
  `)
  const editor = dom.querySelector('godo-editor')

  expect(editor.value).toBe('<p>Initial content</p>')

  // check that the content is processed by the schema before being set
  editor.value = '<invalid-tag>Modified content</invalid-tag>'

  expect(editor.value).toBe('<p>Modified content</p>')
  expect(editor.querySelector('textarea').value).toBe('<p>Modified content</p>')
  expect(editor.querySelector('.ProseMirror').innerHTML).toBe('<p>Modified content</p>')
})

